unit ODBC32;

interface

{ ODBC Classes v1.0 (C)2K, By Paul TOTH <tothpaul@free.fr>
  http://tothpaul.free.fr
}
{
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

Uses
 Windows,Classes,SysUtils,ODBCAPI;

{
  TSQLDriver    = ODBC Driver Connection
   - ListTables()
   - ListFields()
   - Prepare()
   - Execute()
   - ExecDirect()
   - Select()

  TSQLStatement = an SQL Request
   - ResultCols
   - ParamCount
   - Fields
   - Params
   - Execute
   - Fetch
   TSQLFields   = Result Cols of an SQL Request
    - FieldCount
    - Fields[]
   TSQLParams   = Params Collection
    - Add()
    - FieldCount
    - Fields[]
    TSQLField   = One Field in a SQLFields collection
     - Name
     - AsString


  TSQLRecordSet = Set of Records (Block Cursor)
   - FieldCount
   - FieldName[]
   - FieldText[]

 -----------------------------------------------------
  var
   drv:TSQLDriver;
   sql:TSQLStatement;
   fld:TSQLFields;
   rcd:TSQLRecordSet;
     i:integer;
  begin

   drv:=TSQLDriver.Create('DSN=SampleDB');

   sql:=drv.ExecDirect('SELECT * FROM Table'); // select
   fld:=sql.Fields; // Bind fields
   sql.Fetch; // Fetch data
   for i:=0 to fld.FieldCount-1 do ShowMessage(fld.Fields[i].Name+'='+fld.Fields[i].AsString);

   rcd:=drv.Select('SELECT * FROM Table',100); // select per 100 records
   while not rcd.EOF do begin // fetch all the table (not only the 100 first records !)
    for i:=0 to rcd.FieldCount-1 do ShowMessage(rcd.FieldName[i]+'='+dlf.FieldText[i]);
    rcd.Next;
   end;

  end;
}

Type
// Custom ODBC Class
 TCustomSQL=class
 private
  HandleType:SQLSMALLINT;
  Handle    :SQLHANDLE;
  fError    :SQLRETURN;
  fErrorMsg :string;
  procedure SetError(Value:SQLRETURN);
 public
  Constructor Create;
  property Error:SQLRETURN read fError write SetError;
  property ErrorMsg:string read fErrorMsg;
 end;

// forward
 TSQLDriver=class;
 TSQLStatement=class;
 TSQLFields=class;

 TSQLData=class // Custom Class for Fields & Params
  private
   fType:SQLSMALLINT; // SQL TYPE
   fSize:SQLINTEGER;  // Data Size
   fPrec:SQLSMALLINT;
   fNull:SQLSMALLINT;
  public
   Constructor Create; virtual;
   Procedure SetType(AType:SQLSMALLINT;ASize:SQLUINTEGER); virtual;
   property SQLType:SQLSMALLINT read fType;
   property Size:SQLINTEGER  read fSize;
   property Prec:SQLSMALLINT read fPrec write fPrec;
   property Null:SQLSMALLINT read fNull write fNull;
 end;

 TSQLRecord=class(TSQLData) // a Field in a RecordSet
  private
   Offset:integer;
   function AsString(fData:PChar):string;
 end;

 TSQLField=class(TSQLData) //  a stand alone Field
  private
   Index:integer;
   fName:string;
   fBind:pointer;
   fLen :SQLINTEGER;
   Procedure SetType(AType:SQLSMALLINT;ASize:SQLUINTEGER); override;
   function GetAsString:string;
   procedure SetAsString(Value:string);
  public
   Constructor Create; override;
   Destructor Destroy; override;
   Property Name:string read fName;
   Property AsString:string read GetAsString write SetAsString;
  end;

 TCustomFields=class
 private
  fStmt:TSQLStatement;
  fList:TList;
  function GetField(Index:integer):TSQLField;
 public
  Constructor Create(AStmt:TSQLStatement); virtual;
  Destructor Destroy; override;
  function FieldCount:integer;
  function FieldByName(AName:string):TSQLField;
  property Fields[Index:integer]:TSQLField read GetField;
 end;

 TSQLFields=class(TCustomFields) // Statement Fields
 private
  Procedure CreateField(Index:integer);
 public
  Constructor Create(AStmt:TSQLStatement); override;
  Destructor Destroy; override;
  procedure GetData(Index:integer;Data:pointer; Var Size:integer);
  function GetText(Index:integer):string;
 end;

 TSQLParams=class(TCustomFields)
 private
  procedure Bind;
 public
  Destructor Destroy; override;
  function Add(AName:string; AType:SQLSMALLINT; ASize:SQLINTEGER; APrec:SQLSMALLINT):TSQLField;
 end;

 TCustomStmt=class(TCustomSQL)
 private
  fDriver:TSQLDriver;
  fResultCols:SQLSMALLINT;
  fParamCount:SQLSMALLINT;
 public
  Constructor Create(ADriver:TSQLDriver); virtual;
  Destructor Destroy; override;
  property ParamCount:SmallInt read fParamCount;
 end;

 TSQLRecordSet=class(TCustomStmt) // Array of Fields (TSQLRecord)
  private
   fCount :integer;
   fFields:TStringList;
   fSize  :integer;
   fData  :PChar;
   fFetchedRows:SQLSMALLINT;
   fRowStatus:pointer;
   fBOF   :boolean;
   fEOF   :boolean;
   fSelected:integer;
   procedure Select(SQL:string);
   procedure CreateField(Index:integer);
   function Fields(Index:integer):TSQLRecord;
   function GetFieldName(index:integer):string;
   function GetFieldText(index:integer):string;
   function GetCell(ACol,ARow:integer):string;
  public
   Constructor Create(ADriver:TSQLDriver); override;
   Destructor Destroy; override;
   function CacheNext(Count:integer):boolean;
   function CachePrev(Count:integer):boolean;
   procedure First;
   procedure Next;
   procedure Prior;
   procedure Last;
   function BOF:boolean;
   function EOF:boolean;
   function LastRecord:boolean;
   Procedure Refresh;
   function FieldCount:integer;
   function Status(ARow:integer):integer;
   property FieldName[Index:integer]:string read GetFieldName;
   property FieldText[Index:integer]:string read GetFieldText;
   property Cells[ACol,ARow:integer]:string read GetCell;
   property FetchedCount:SmallInt read fFetchedRows;
   property Selected:integer read fSelected write fSelected;
   property CacheBOF:boolean read fBOF;
   property CacheEOF:boolean read fEOF;
 end;

 TSQLStatement=class(TCustomStmt) // SQL Statement
 private
  fFields:TSQLFields;
  fParams:TSQLParams;
  procedure Prepare(SQL:string);
  procedure ExecDirect(SQL:string);
 public
  Constructor Create(ADriver:TSQLDriver); override;
  Destructor Destroy; override;
  procedure Execute;
  function RowCount:integer;
  function Fetch:boolean;
  function Fields:TSQLFields;
  function Params:TSQLParams;
  property ResultCols:SmallInt read fResultCols;
  property ParamCount:SmallInt read fParamCount;
 end;

 TSQLDriver=class(TCustomSQL) // ODBC Driver
 private
  fConnectString:string;
 public
  constructor Create(dsn:string);
  destructor Destroy; override;
  function AllocStmt:SQLHSTMT;
  property ConnectString:string read fConnectString;
  procedure ListTables(List:TStrings);
  procedure ListFields(Table:string; List:TStrings);
  function Prepare(SQL:string):TSQLStatement;
  function Execute(SQL:string):TSQLStatement;
  function Select(SQL:string; CacheSize:integer):TSQLRecordSet;
 end;

function SQLTypeName(fType:integer):string;
function SQLNameType(fName:string):integer;

Var
 DebugInfo:integer=0;

implementation

Var
 API:string;

function SQLTypeName(fType:integer):string;
 begin
  case fType of
   SQL_CHAR          : Result:='String';
   SQL_SMALLINT      : Result:='SmallInt';
   SQL_INTEGER       : Result:='Integer';
   SQL_DOUBLE        : Result:='Double';
   SQL_TYPE_DATE     : Result:='Date';
   SQL_TYPE_TIMESTAMP: Result:='DateTime';
   SQL_BINARY        : Result:='Memo';
   else                Result:='Unknow';
  end;
 end;

function SQLNameType(fName:string):integer;
 begin
  case fName[2] of
   't':Result:=SQL_CHAR;
   'm':Result:=SQL_SMALLINT;
   'n':if fName[1]='I' then Result:=SQL_INTEGER else Result:=SQL_UNKNOWN_TYPE;
   'o':Result:=SQL_DOUBLE;
   'a':if Length(fName)=4 then Result:=SQL_TYPE_DATE else Result:=SQL_TYPe_TIMESTAMP;
   'e':Result:=SQL_BINARY;
   else Result:=SQL_UNKNOWN_TYPE;
  end;
 end;
//----------------------------------------------------------------------------//
Constructor TCustomSQL.Create;
 begin
  inherited Create; // not very usefull
  Handle:=SQL_NULL_HANDLE;
  fError:=0;
  fErrorMsg:='';
 end;

procedure TCustomSQL.SetError(Value:SQLRETURN);
var
 Index:SQLSMALLINT;
 MessageText:ShortString;
 TextLength:SQLSMALLINT;
 SqlState:array[0..6] of char;
 NativeError:SQLINTEGER;
begin
 if DebugInfo>1 then MessageBox(0,PChar(API + ' - '+IntToStr(Value)),'Debug',0);
 fError:=Value;
 fErrorMsg:='';
 Index:=1;
 While SQLGetDiagRec(HandleType,Handle,Index,SqlState,NativeError,@MessageText[1],255,TextLength)in [0,1] do begin
  MessageText[0]:=Chr(TextLength);
  fErrorMsg:=fErrorMsg+SqlState+':'+MessageText+#13#10;
  inc(Index);
 end;
 if fError<>0 then begin
  if fErrorMsg='' then fErrorMsg:='ODBC Error #'+IntToStr(fError);
  if (fError=1)and(DebugInfo>0) then MessageBox(0,pchar(fErrorMsg),PChar(API),0);
  API:='';
  if fError<>1 then raise Exception.Create(fErrorMsg);
 end;
 API:='';
end;

//----------------------------------------------------------------------------//
Constructor TSQLData.Create;
 begin
  inherited Create;
  fType:=0;
  fSize:=0;
  fPrec:=0;
  fNull:=0;
 end;

procedure TSQLData.SetType(AType:SQLSMALLINT;ASize:SQLUINTEGER);
 begin
  case AType of
   SQL_CHAR,SQL_VARCHAR: begin
    AType:=SQL_CHAR;
    inc(ASize); { for Null Terminate String }
   end;
   SQL_TINYINT, SQL_SMALLINT: begin
    AType:=SQL_SMALLINT;
    ASize:=SizeOf(SQLSMALLINT);
   end;
   SQL_INTEGER: begin
    ASize:=SizeOf(SQLINTEGER);
   end;
   SQL_DECIMAL, SQL_NUMERIC, SQL_REAL, SQL_FLOAT, SQL_DOUBLE: begin
    AType:=SQL_DOUBLE;
    ASize:=SizeOf(SQLDOUBLE);
   end;
   SQL_DATE, SQL_TYPE_DATE: begin
    AType:=SQL_TYPE_DATE;
    ASize:=SizeOf(SQLDATE);
   end;
   SQL_TIME, SQL_TYPE_TIME: begin
    AType:=SQL_TYPE_TIME;
    ASize:=SizeOf(SQLTIME);
   end;
   SQL_TIMESTAMP, SQL_TYPE_TIMESTAMP: begin
    AType:=SQL_TYPE_TIMESTAMP;
    ASize:=SizeOf(SQLTIMESTAMP);
   end;
   SQL_LONGVARCHAR : begin
    AType:=SQL_BINARY;
    ASize:=0;
   end;
   else raise Exception.Create('Unsupported data type #'+IntToStr(AType));
  end;
  fType:=AType;
  fSize:=ASize;
 end;
//----------------------------------------------------------------------------//
function GetSQLString(fType:SQLSMALLINT; fBind:pointer; fSize:SQLUINTEGER):string;
 begin
  if fType=SQL_BINARY then
   Result:='(Blob)'
  else begin
   Result:='';
   if fSize>0 then begin
    case fType of
     SQL_CHAR          : Result:=PChar(fBind);
     SQL_SMALLINT      : Result:=IntToStr(SmallInt(fBind^));
     SQL_INTEGER       : Result:=IntToStr(Integer(fBind^));
     SQL_DOUBLE        : Result:=FloatToStr(Double(fBind^));
     SQL_TYPE_DATE     : with SQLDATE(fBind^) do try
                          Result:=DateToStr(EncodeDate(Year,Month,Day));
                         except
                          Result:='';
                         end;
     SQL_TYPE_TIMESTAMP: with SQLTIMESTAMP(fBind^) do try
                          Result:=DateTimeToStr(EncodeDate(Year,Month,Day)+EncodeTime(Hour,Minute,Second,Fraction));
                         except
                          Result:='';
                         end;
     else raise Exception.Create('unsupported type #'+IntToStr(fType)+' in ODCB32.GetSQLString()');
    end;
   end;  
  end;
 end;

procedure SetSQLString(fType:SQLSMALLINT; fBind:pointer; fSize:SQLUINTEGER; Value:string; Var fLen:SQLUINTEGER);
 var
  date :TDateTime;
  fract:word;
 begin
  case fType of
   SQL_CHAR          : begin StrPLCopy(PChar(fBind),Value,fSize); fLen:=StrLen(PChar(fBind)); end;
   SQL_SMALLINT      : SmallInt(fBind^):=StrToInt(Value);
   SQL_INTEGER       : Integer(fBind^) :=StrToInt(Value);
   SQL_DOUBLE        : Double(fBind^)  :=StrToFloat(Value);
   SQL_TYPE_DATE     : with SQLDATE(fBind^) do DecodeDate(StrToDateTime(Value),Year,Month,Day);
   SQL_TYPE_TIMESTAMP: with SQLTIMESTAMP(fBind^) do begin
                        date:=StrToDateTime(Value);
                        DecodeDate(date,Year,Month,Day);
                        DecodeTime(date,Hour,Minute,Second,Fract);
                        Fraction:=Fract;
                       end;
   else raise Exception.Create('unsupported type #'+IntToStr(fType)+' in ODCB32.SetSQLString()');
  end;
 end;
//----------------------------------------------------------------------------//
function TSQLRecord.AsString(fData:PChar):string;
 var
  i:^SQLINTEGER;
 begin
  i:=@fData[Offset+fSize];
  Result:=GetSQLString(fType,@fData[Offset],i^);
 end;

//----------------------------------------------------------------------------//
Constructor TSQLField.Create;
 begin
  inherited Create; // TSQLData
  fBind:=nil;
  fLen :=0;
 end;

Destructor TSQLField.Destroy;
 begin
  if fBind<>nil then FreeMem(fBind);
  inherited Destroy;
 end;

procedure TSQLField.SetType(AType:SQLSMALLINT;ASize:SQLUINTEGER);
 begin
  inherited SetType(AType,ASize);
  if fBind<>nil then FreeMem(fBind);
  if fSize=0 then fBind:=nil else fBind:=AllocMem(fSize);
 end;

Procedure TSQLField.SetAsString(Value:string);
 begin
  if fBind<>nil then SetSQLString(fType,fBind,fSize,Value,fLen);
 end;

Function TSQLField.GetAsString:string;
 begin
  if fBind=nil then Result:='' else Result:=GetSQLString(fType,fBind,fLen);
 end;

//----------------------------------------------------------------------------//
Constructor TCustomFields.Create(AStmt:TSQLStatement);
 begin
  inherited Create; // none
  fStmt:=AStmt;
  fList:=TList.create;
 end;

Destructor TCustomFields.Destroy;
 var
  i:integer;
 begin
  for i:=0 to fList.Count-1 do Fields[i].Free;
  fList.Free;
  inherited Destroy;
 end;

function TCustomFields.GetField(Index:integer):TSQLField;
 begin
  Result:=TSQLField(fList[Index]);
 end;

function TCustomFields.FieldCount:integer;
 begin
  Result:=fList.Count;
 end;

function TCustomFields.FieldByName(AName:string):TSQLField;
 var
  i:integer;
 begin
  for i:=0 to FieldCount-1 do begin
   Result:=Fields[i];
   if Result.Name=AName then exit;
  end;
  raise exception.create('Field '+AName+' not found');
 end;

//----------------------------------------------------------------------------//
Constructor TSQLFields.Create(AStmt:TSQLStatement);
 var
  i:integer;
 begin
  inherited Create(AStmt); // TCustomFields
  for i:=1 to fStmt.ResultCols do CreateField(i);
 end;

Destructor TSQLFields.Destroy;
 begin
  SQLFreeStmt(fStmt.Handle,SQL_UNBIND);
  fStmt.fFields:=nil;
  inherited Destroy;
 end;

procedure TSQLFields.CreateField(Index:integer);
 var
  cName    :ShortString;
  cNameSize:SQLSMALLINT;
  cType    :SQLSMALLINT;
  cSize    :SQLUINTEGER;
  cPrec    :SQLSMALLINT;
  cNull    :SQLSMALLINT;
  Field    :TSQLField;
 begin
  cNameSize:=255;
  with fStmt do Error:=SQLDescribeCol(Handle,SQLSMALLINT(Index),SQLCHAR(@cName[1]),cNameSize,cNameSize,cType,cSize,cPrec,cNull);
  cName[0]:=Chr(cNameSize);
  Field:=TSQLField.Create;
  fList.Add(Field);
  Field.Index:=Index;
  Field.fName:=cName;
  Field.fPrec:=cPrec;
  Field.fNull:=cNull;
  Field.SetType(cType,cSize);
//  if Field.fSize<cSize then raise exception.Create('Invalide type conversion for '+cName+':'+SQLTypeName(Field.fType)+' '+IntToStr(cSize)+'->'+IntToStr(Field.fSize));
  with Field do if fSize>0 then
   fStmt.Error:=SQLBindCol(fStmt.Handle,SQLSMALLINT(Index),fType,fBind,fSize,fLen);
 end;

procedure TSQLFields.GetData(Index:integer; Data:pointer; Var Size:integer);
 begin
  if Data=nil then begin
   fStmt.Error:=SQLGetData(fStmt.Handle,SQLSMALLINT(Index+1),SQL_CHAR,@Size,0,Size);
  end else begin
   fStmt.Error:=SQLGetData(fStmt.Handle,SQLSMALLINT(Index+1),SQL_CHAR,Data,Size,Size);
  end;
 end;

function TSQLFields.GetText(Index:integer):string;
 var
  field:TSQLField;
  size:integer;
 begin
  field:=Fields[Index];
  if Field.fType<>SQL_BINARY then
   Result:=Field.AsString
  else begin
   GetData(Index,nil,Size);
   if Size<=0 then
    Result:=''
   else begin
    Inc(Size);
    SetLength(Result,Size);
    GetData(Index,@Result[1],Size);
   end;
  end;
 end;

//----------------------------------------------------------------------------//
Destructor TSQLParams.Destroy;
 begin
  SQLFreeStmt(fStmt.Handle,SQL_RESET_PARAMS);
  fStmt.fParams:=nil;
  inherited Destroy;
 end;

function TSQLParams.Add(AName:string; AType:SQLSMALLINT; ASize:SQLINTEGER; APrec:SQLSMALLINT):TSQLField;
 begin
  Result:=TSQLField.Create;
  fList.Add(Result);
  Result.Index:=fList.Count;
  Result.fName:=AName;
  Result.fPrec:=APrec;
  Result.fNull:=1;
  Result.SetType(AType,ASize);
 end;

procedure TSQLParams.Bind;
 var
  i:integer;
 begin
  for i:=0 to FieldCount-1 do
   with Fields[i] do
    fStmt.Error:=SQLBindParameter(fStmt.Handle,Index,SQL_PARAM_INPUT,fType,fType,fSize,fPrec,fBind,fSize,fLen);
 end;
//----------------------------------------------------------------------------//
Constructor TCustomStmt.Create(ADriver:TSQLDriver);
 begin
  inherited Create; // TCustomSQL
  fDriver:=ADriver;
  HandleType:=SQL_HANDLE_STMT;
  Handle:=fDriver.AllocStmt;
  fResultCols:=0;
  fParamCount:=0;
 end;

Destructor TCustomStmt.Destroy;
 begin
  SQLCloseCursor(Handle);
  SQLFreeHandle(SQL_HANDLE_STMT,Handle);
  inherited Destroy;
 end;

//----------------------------------------------------------------------------//
Constructor TSQLRecordSet.Create(ADriver:TSQLDriver);
 begin
  inherited Create(ADriver); // TCustomStmt
  fData:=nil;
  fRowStatus:=nil;
  fFields:=TStringList.Create;
 end;

Destructor TSQLRecordSet.Destroy;
 var
  i:integer;
 begin
  for i:=0 to fFields.Count-1 do Fields(i).Free;
  fFields.Free;
  if fData<>nil then FreeMem(fData);
  if fRowStatus<>nil then FreeMem(fRowStatus);
  inherited Destroy;
 end;

procedure TSQLRecordSet.Select(SQL:String);
 var
  i:integer;
 begin
  API:='SQLSetStmtAttr(CURSOR_TYPE)';
  Error:=SQLSetStmtAttr(Handle,SQL_ATTR_CURSOR_TYPE,SQLPOINTER(SQL_CURSOR_DYNAMIC),SQL_IS_INTEGER);
  API:='SQLExecDirect()';
  Error:=SQLExecDirect(Handle,pchar(SQL),SQL_NTS);
  API:='SQLNumResultCols()';
  Error:=SQLNumResultCols(Handle,fResultCols);
  if fResultCols=0 then exit;
  fSize:=0; for i:=1 to fResultCols do CreateField(i);
  GetMem(fData,fCount*fSize);
  API:='SQLSetStmtAttr(BIND_TYPE)';
  Error:=SQLSetStmtAttr(Handle,SQL_ATTR_ROW_BIND_TYPE,SQLPOINTER(fSize),0);
  API:='SQLSetStmtAttr(ARRAY_SIZE)';
  Error:=SQLSetStmtAttr(Handle,SQL_ATTR_ROW_ARRAY_SIZE,SQLPOINTER(fCount),0);
  API:='SQLSetStmtAttr(FETCHED_PTR)';
  Error:=SQLSetStmtAttr(Handle,SQL_ATTR_ROWS_FETCHED_PTR,@fFetchedRows,0);
  fRowStatus:=AllocMem(fCount*SizeOf(SQLUSMALLINT));
  API:='SQLSetStmtAttr(STATUS_PTR)';
  Error:=SQLSetStmtAttr(Handle,SQL_ATTR_ROW_STATUS_PTR,fRowStatus,0);
  for i:=0 to fFields.Count-1 do
   with Fields(i) do if fSize>0 then begin
    API:='SQLBindCol('+fFields[i]+')';
    Error:=SQLBindCol(Handle,SQLSMALLINT(i+1),fType,@fData[Offset],fSize,fData[Offset+fSize]);
   end;
  First;
 end;

Procedure TSQLRecordSet.CreateField(Index:integer);
 var
  cName    :ShortString;
  cNameSize:SQLSMALLINT;
  cType    :SQLSMALLINT;
  cSize    :SQLUINTEGER;
  cPrec    :SQLSMALLINT;
  cNull    :SQLSMALLINT;
  Rcd:TSQLRecord;
 begin
  cNameSize:=255;
  Error:=SQLDescribeCol(Handle,SQLSMALLINT(Index),SQLCHAR(@cName[1]),cNameSize,cNameSize,cType,cSize,cPrec,cNull);
  cName[0]:=Chr(cNameSize);
  Rcd:=TSQLRecord.Create;
  fFields.AddObject(cName,Rcd);
  Rcd.SetType(cType,cSize);
  Rcd.fPrec:=cPrec;
  Rcd.fNull:=cNull;
  Rcd.Offset:=fSize;
  if Rcd.fSize>0 then inc(fSize,Rcd.fSize+SizeOf(SQLINTEGER));
 end;

function TSQLRecordSet.Fields(index:integer):TSQLRecord;
 begin
  Result:=TSQLRecord(fFields.Objects[Index]);
 end;

function TSQLRecordSet.CacheNext(Count:integer):boolean;
 var
  Code:integer;
 begin
  fBOF:=False;
  Result:=False;
  Code:=SQLFetchScroll(Handle,SQL_FETCH_RELATIVE,Count);
  case Code of
   0,1:Result:=True;  // OK, OK with Info
   100:fEOF:=True; // EOF
  else Error:=Code;
  end;
  fEOF:=(Result=False)or(fFetchedRows<fCount);
 end;

function TSQLRecordSet.CachePrev(Count:integer):boolean;
 var
  Code:integer;
 begin
  fEOF:=False;
  Result:=False;
  Code:=SQLFetchScroll(Handle,SQL_FETCH_RELATIVE,-Count);
  case Code of
   0: Result:=True;  // OK
   1: First; // we've reach top of file, how can I keep current pos ??
   100:begin
    Result:=True; fBOF:=True; end; // BOF
  else Error:=Code;
  end;
 end;

procedure TSQLRecordSet.First;
 var
  Err:integer;
 begin
  if BOF then Exit;
  if not fBOF then begin
   API:='SQLFetchScroll(FIRST)';
   Err:=SQLFetchScroll(Handle, SQL_FETCH_FIRST, 0);
   If Err=100 then fFetchedRows:=0 else Error:=Err;
   fBOF:=True;
   fEOF:=(fFetchedRows<fCount);
  end;
  fSelected:=0;
 end;

procedure TSQLRecordSet.Next;
 var
  delta:integer;
 begin
  if EOF then Exit;
  if fSelected<fCount-1 then begin // internal scroll
   inc(fSelected);
  end else begin // cache scroll
   delta:=fCount*2 div 3;
   if CacheNext(delta) then Dec(fSelected,Delta-1);
  end;
 end;

procedure TSQLRecordSet.Prior;
 var
  delta:integer;
 begin
  if BOF then exit;
  if fSelected>0 then begin // internal scroll
   dec(fSelected);
  end else begin
   delta:=fCount*2 div 3;
   if CachePrev(delta) then Inc(fSelected,delta-1);
  end;
 end;

procedure TSQLRecordSet.Last;
 begin
  if EOF then exit;
  if not fEOF then begin
   Error:=SQLFetchScroll(Handle, SQL_FETCH_LAST, 0);
   fBOF:=False;
   fEOF:=True;
  end;
  fSelected:=fFetchedRows-1;
 end;

function TSQLRecordSet.BOF:boolean;
 begin
  result:=fBOF and (fSelected=0);
 end;

function TSQLRecordSet.EOF:boolean;
 begin
  result:=fEOF and (fSelected>fFetchedRows-1);
 end;

function TSQLRecordSet.LastRecord:boolean;
 begin
  result:=fEOF and (fSelected=fFetchedRows-1);
 end;

Procedure TSQLRecordSet.Refresh;
 begin
  Error:=SQLFetchScroll(Handle,SQL_FETCH_RELATIVE,0);
 end;

function TSQLRecordSet.FieldCount:integer;
 begin
  result:=fResultCols;
 end;

function TSQLRecordSet.Status(ARow:integer):integer;
 var
  i:^SQLUSMALLINT;
 begin
  i:=fRowStatus;
  inc(i,ARow);
  Result:=i^;
 end;

function TSQLRecordSet.GetFieldName(index:integer):string;
 begin
  Result:=fFields[Index];
 end;

function TSQLRecordSet.GetFieldText(index:integer):string;
 begin
  Result:=Fields(Index).AsString(@fData[fSelected*fSize]);
 end;

function TSQLRecordSet.GetCell(ACol,ARow:integer):string;
 begin
  Result:=Fields(ACol).AsString(@fData[ARow*fSize]);
 end;

//----------------------------------------------------------------------------//
Constructor TSQLStatement.Create(ADriver:TSQLDriver);
 begin
  inherited Create(ADriver);
  fFields:=nil;
  fParams:=nil;
 end;

Destructor TSQLStatement.Destroy;
 begin
  if fFields<>nil then fFields.Free;
  if fParams<>nil then fParams.Free;
  inherited Destroy;
 end;

procedure TSQLStatement.Prepare(SQL:string);
 begin
  if fFields<>nil then begin
   fFields.Free;
   fFields:=nil;
  end;
  if fParams<>nil then begin
   fParams.Free;
   fParams:=nil;
  end;
  API:='SQLPrepare()';
  Error:=SQLPrepare(Handle,pchar(SQL),SQL_NTS);
  API:='SQLNumResultCols()';
  Error:=SQLNumResultCols(Handle,fResultCols);
  API:='SQLNumParams()';
  Error:=SQLNumParams(Handle,fParamCount);
 end;

procedure TSQLStatement.Execute;
 begin
  Error:=SQLFreeStmt(Handle,SQL_CLOSE);
  if fParams<>nil then fParams.Bind;
  API:='SQLExecute()';
  Error:=SQLExecute(Handle);
 end;

function TSQLStatement.RowCount:integer;
 begin
  API:='SQLRowCount()';
  Error:=SQLRowCount(Handle,SQLInteger(Result));
 end;

procedure TSQLStatement.ExecDirect(SQL:string);
 begin
  if fFields<>nil then begin
   fFields.Free;
   fFields:=nil;
  end;
  API:='SQLExecDirect()';
  Error:=SQLExecDirect(Handle,pchar(SQL),SQL_NTS);
  API:='SQLNumResultCols()';
  Error:=SQLNumResultCols(Handle,fResultCols);
  API:='SQLNumParams()';
  Error:=SQLNumParams(Handle,fParamCount);
 end;

function TSQLStatement.Fields:TSQLFields;
 begin
  if (fFields=nil)and(fResultCols>0) then fFields:=TSQLFields.Create(Self);
  Result:=fFields;
 end;

function TSQLStatement.Params:TSQLParams;
 begin
  if (fParams=nil)and(fParamCount>0) then fParams:=TSQLParams.Create(Self);
  Result:=fParams;
 end;

function TSQLStatement.Fetch:boolean;
 var
  err:integer;
 begin
  API:='SQLFetch()';
  Err:=SQLFetch(Handle);
  Result:=(Err<>100);
  if Result then Error:=Err;
 end;

//----------------------------------------------------------------------------//

Constructor TSQLDriver.Create(dsn:string);
 var
  cnx:array[0..1024] of char;
  cnxsize:SQLSMALLINT;
 begin
  inherited Create; // TCustomSQL
  HandleType:=SQL_HANDLE_ENV;
  Handle:=ODBCHandle;
  fConnectString:='';
  API:='SQLAllocHandle(DBC)';
  Error:=SQLAllocHandle(SQL_HANDLE_DBC,ODBCHandle,Handle);
  HandleType:=SQL_HANDLE_DBC;
  cnxsize:=1024;
  API:='SQLDriverConnect()';
  Error:=SQLDriverConnect(
   Handle,GetFocus,
   pchar(dsn),Length(dsn),
   cnx,cnxsize,cnxsize,1
  );
  cnx[cnxSize]:=#0;
  fConnectString:=cnx;
 end;

Destructor TSQLDriver.Destroy;
 begin
  if Handle<>SQL_NULL_HANDLE then begin
   SQLDisconnect(Handle);
   SQLFreeHandle(SQL_HANDLE_DBC,Handle);
  end;
  inherited Destroy;
 end;

function TSQLDriver.AllocStmt:SQLHSTMT;
 begin
  API:='SQLAllocHandle(STMT)';
  Error:=SQLAllocHandle(SQL_HANDLE_STMT,Handle,Result);
 end;

procedure TSQLDriver.ListTables(List:TStrings);
 Var
  HStmt:SQLHSTMT;
  cName:PChar; cSize,rcSize:SQLUINTEGER;
  oName:PChar; oSize,roSize:SQLUINTEGER;
  Dummy:SQLSMALLINT;
 begin
  List.Clear;
  HStmt:=AllocStmt;
  try
  if SQLTables(HStmt,nil,0,nil,0,nil,0,nil,0) in [0,1] then begin
   SQLDescribeCol(HStmt,2,nil,0,Dummy,Dummy,oSize,Dummy,Dummy); GetMem(oName,oSize);
   SQLDescribeCol(HStmt,3,nil,0,Dummy,Dummy,cSize,Dummy,Dummy); GetMem(cName,cSize);
   SQLBindCol(HStmt,2,sql_char,oName,oSize,roSize);
   SQLBindCol(HStmt,3,sql_char,cName,cSize,rcSize);
   while SQLFetch(HStmt)=0 do begin
     while (roSize>1)and(oName[roSize-1]=' ') do dec(roSize); oName[roSize]:=#0;
     while (rcSize>1)and(cName[rcSize-1]=' ') do dec(rcSize); cName[rcSize]:=#0;
     if StrComp(oName,'SYS')<>0 then begin
      if oName^<>#0 then
       List.Add(StrPas(oName)+'.'+StrPas(cName))
      else
       List.Add(StrPas(cName))
     end;
   end;
   FreeMem(cName,cSize);
   FreeMem(oName,oSize);
  end;
  finally
   SQLFreeHandle(SQL_HANDLE_STMT,HStmt);
  end;
 end;

procedure TSQLDriver.ListFields(Table:string; List:TStrings);
 Var
  HStmt:SQLHSTMT;//THandle32;
  cName:ShortString;  cSize,rSize:SQLUINTEGER; { Column Name, size and readed size }
  Dummy:SQLSMALLINT;
 begin
  List.Clear;
  HStmt:=AllocStmt;
  try
   if SQLColumns(HStmt,nil,0,nil,0,PChar(Table),SQL_NTS,nil,0) in [0,1] then begin
    SQLDescribeCol(HStmt,4,nil,0,Dummy,Dummy,cSize,Dummy,Dummy);
    SQLBindCol(HStmt,4,sql_char,@cName[1],cSize,rSize);
    while SQLFetch(HStmt)=0 do begin
      while (rSize>1)and(cName[rSize-1]=' ') do dec(rSize);
      cName[0]:=Chr(rSize);
      List.Add(cName);
    end;
   end;
  finally
   SQLFreeHandle(SQL_HANDLE_STMT,HStmt);
  end;
 end;

function TSQLDriver.Prepare(SQL:string):TSQLStatement;
 begin
  Result:=TSQLStatement.Create(Self);
  Result.Prepare(SQL);
 end;

function TSQLDriver.Execute(SQL:string):TSQLStatement;
 begin
  Result:=TSQLStatement.Create(Self);
  Result.ExecDirect(SQL);
 end;

function TSQLDriver.Select(SQL:string;CacheSize:integer):TSQLRecordSet;
 begin
  Result:=TSQLRecordSet.Create(Self);
  Result.fCount:=CacheSize;
  Result.Select(SQL);
 end;

end.
