unit ODBCAPI;

interface

Type
 SQLSMALLINT =smallint;
 SQLUSMALLINT=word;
 SQLCHAR     =pchar;
 SQLINTEGER  =LongInt;
 SQLUINTEGER =LongInt;
 SQLHANDLE   =LongInt;
 SQLPOINTER  =Pointer;
 SQLHWND     =Integer;
 SQLDOUBLE   =Double;
 SQLFLOAT    =Double;

 SQLHENV    =SQLHANDLE;
 SQLHDBC    =SQLHANDLE;
 SQLHSTMT   =SQLHANDLE;

 SQLRETURN  =SQLSMALLINT;

 SQLDATE = packed record
  Year  : SQLUSMALLINT;
  Month : SQLUSMALLINT;
  Day   : SQLUSMALLINT;
 end;

 SQLTIME = packed record
  Hour   : SQLUSMALLINT;
  Minute : SQLUSMALLINT;
  Second : SQLUSMALLINT;
 end;

 SQLTIMESTAMP = packed record
  Year :     SQLUSMALLINT;
  Month :    SQLUSMALLINT;
  Day :      SQLUSMALLINT;
  Hour :     SQLUSMALLINT;
  Minute :   SQLUSMALLINT;
  Second :   SQLUSMALLINT;
  Fraction : SQLUINTEGER;
 end;

Const
 SQL_NTS = (-3);

Var
 ODBCHandle:SQLHANDLE;

// Error Diagnostic ---------------------------------------------
Function SQLGetDiagRec(
     HandleType    :SQLSMALLINT;
     Handle        :SQLHANDLE;
     RecNumber     :SQLSMALLINT;
     Sqlstate      :SQLCHAR;
 Var NativeErrorPtr:SQLINTEGER;
     MessageText   :SQLCHAR;
     BufferLength  :SQLSMALLINT;
 Var TextLengthPtr :SQLSMALLINT
):SQLRETURN; stdcall;

// SQLAllocHandle -----------------------------------------------
Const
 SQL_NULL_HANDLE  = 0;
 SQL_HANDLE_ENV   = 1;
 SQL_HANDLE_DBC   = 2;
 SQL_HANDLE_STMT  = 3;
 SQL_HANDLE_DESC  = 4;

Function SQLAllocHandle(
     HandleType:SQLSMALLINT;
     InputHandle:SQLHANDLE;
 Var OutputHandlePtr:SQLHANDLE
):SQLRETURN; stdcall;

Function SQLFreeHandle(
     HandleType:SQLSMALLINT;
     Handle    :SQLHANDLE
):SQLRETURN; stdcall;

Const
 SQL_CLOSE=0;
 SQL_DROP=1;
 SQL_UNBIND=2;
 SQL_RESET_PARAMS=3;

Function SQLFreeStmt(
     StatementHandle:SQLHSTMT;
     Option         :SQLUSMALLINT
):SQLRETURN; stdcall;

// Environment -----------------------------------------------------
Const
 SQL_ATTR_ODBC_VERSION = 200;
  SQL_OV_ODBC2	       = SQLPOINTER(2);
  SQL_OV_ODBC3	       = SQLPOINTER(3);

Function SQLSetEnvAttr(
      EnvironmentHandle:SQLHENV;
      Attribute        :SQLINTEGER;
      ValuePtr         :SQLPOINTER;
      StringLength     :SQLINTEGER
):SQLRETURN; stdcall;

// Connection -----------------------------------------------------
Const
 SQL_ATTR_ODBC_CURSORS = 110;
  SQL_CUR_USE_IF_NEEDED = SQLPOINTER(0);
  SQL_CUR_USE_ODBC      = SQLPOINTER(1);
  SQL_CUR_USE_DRIVER    = SQLPOINTER(2);

Function SQLSetConnectAttr(
      ConnectionHandle:SQLHDBC;
      Attribute       :SQLINTEGER;
      ValuePtr        :SQLPOINTER;
      StringLength    :SQLINTEGER
):SQLRETURN; stdcall;

// Driver ---------------------------------------------------------
Function SQLDriverConnect(
     ConnectionHandle   :SQLHDBC;
     WindowHandle       :SQLHWND;
     InConnectionString :SQLCHAR;
     StringLength1      :SQLSMALLINT;
     OutConnectionString:SQLCHAR;
     BufferLength       :SQLSMALLINT;
 var StringLength2Ptr   :SQLSMALLINT;
     DriverCompletion   :SQLUSMALLINT
):SQLRETURN; stdcall;

Function SQLDisconnect(
     ConnectionHandle:SQLHDBC
):SQLRETURN; stdcall;

// Statement ------------------------------------------------------
Const
 SQL_ATTR_ROW_BIND_TYPE	   = 5;
  SQL_BIND_BY_COLUMN = SQLPOINTER(0);

 SQL_ATTR_CURSOR_TYPE      = 6;
  SQL_CURSOR_FORWARD_ONLY     = SQLPOINTER(0);
  SQL_CURSOR_KEYSET_DRIVEN    = SQLPOINTER(1);
  SQL_CURSOR_DYNAMIC          = SQLPOINTER(2);
  SQL_CURSOR_STATIC           = SQLPOINTER(3);

 SQL_ATTR_CONCURRENCY	   = 7;
  SQL_CONCUR_READ_ONLY = SQLPOINTER(1);
  SQL_CONCUR_LOCK      = SQLPOINTER(2);
  SQL_CONCUR_ROWVER    = SQLPOINTER(3);
  SQL_CONCUR_VALUES    = SQLPOINTER(4);
  SQL_CONCUR_DEFAULT   = SQL_CONCUR_READ_ONLY; { Default value }


 SQL_ATTR_USE_BOOKMARKS    = 12;
  SQL_UB_OFF      = SQLPOINTER(0);
  SQL_UB_ON       = SQLPOINTER(1);
  SQL_UB_VARIABLE = SQLPOINTER(2);

 SQL_ATTR_ROW_STATUS_PTR   = 25;
 SQL_ATTR_ROWS_FETCHED_PTR = 26;
 SQL_ATTR_ROW_ARRAY_SIZE   = 27;

 SQL_IS_POINTER    = (-4);
 SQL_IS_UINTEGER   = (-5);
 SQL_IS_INTEGER    = (-6);
 SQL_IS_USMALLINT  = (-7);
 SQL_IS_SMALLINT   = (-8);

Function SQLSetStmtAttr(
      StatementHandle:SQLHSTMT;
      Attribute      :SQLINTEGER;
      ValuePtr       :SQLPOINTER;
      StringLength   :SQLINTEGER
):SQLRETURN; stdcall;

// Bind Columns ----------------------------------------
Const
 { SQL data type codes }
  SQL_BIT           =(-7);
  SQL_TINYINT       =(-6);
  SQL_BIGINT        =(-5);
  SQL_LONGVARBINARY =(-4);
  SQL_VARBINARY     =(-3);
  SQL_BINARY        =(-2);
  SQL_LONGVARCHAR   =(-1);
  SQL_UNKNOWN_TYPE  = 0;
  SQL_CHAR          = 1;
  SQL_NUMERIC       = 2;
  SQL_DECIMAL       = 3;
  SQL_INTEGER       = 4;
  SQL_SMALLINT      = 5;
  SQL_FLOAT         = 6;
  SQL_REAL          = 7;
  SQL_DOUBLE        = 8;
  SQL_DATE          = 9;
  SQL_DATETIME      = 9;
  SQL_TIME          = 10;
  SQL_TIMESTAMP     = 11;
  SQL_VARCHAR       = 12;
  SQL_TYPE_DATE     = 91;
  SQL_TYPE_TIME     = 92;
  SQL_TYPE_TIMESTAMP= 93;

type
  SQL_DATE_STRUCT = packed record
    Year  : SQLSMALLINT;
    Month : SQLUSMALLINT;
    Day   : SQLUSMALLINT;
  end;
  PSQL_DATE_STRUCT = ^SQL_DATE_STRUCT;

  SQL_TIME_STRUCT = packed record
    Hour : SQLUSMALLINT;
    Minute : SQLUSMALLINT;
    Second : SQLUSMALLINT;
  end;
  PSQL_TIME_STRUCT = ^SQL_TIME_STRUCT;

  SQL_TIMESTAMP_STRUCT = packed record
    Year :     SQLUSMALLINT;
    Month :    SQLUSMALLINT;
    Day :      SQLUSMALLINT;
    Hour :     SQLUSMALLINT;
    Minute :   SQLUSMALLINT;
    Second :   SQLUSMALLINT;
    Fraction : SQLUINTEGER;
  end;
  PSQL_TIMESTAMP_STRUCT = ^SQL_TIMESTAMP_STRUCT;
(*
Const
 SQL_CHAR     = 1;
 SQL_NUMERIC  = 2;
 SQL_DECIMAL  = 3;
 SQL_INTEGER  = 4;
 SQL_SMALLINT = 5;
 SQL_FLOAT    = 6;
 SQL_REAL     = 7;
 SQL_DOUBLE   = 8;
 SQL_DATE1    = 9; // AS/400 ?
 SQL_DATE     =11; // Access ?
 SQL_VARCHAR  =12;
 SQL_TIMESTAMP=91; // AS/400 ?
 SQL_DATETIME =93; // ???
*)
Function SQLDescribeCol(
     StatementHandle :SQLHSTMT;
     ColumnNumber    :SQLSMALLINT;
     ColumnName      :SQLCHAR;
     BufferLength    :SQLSMALLINT;
 Var NameLengthPtr   :SQLSMALLINT;
 Var DataTypePtr     :SQLSMALLINT;
 Var ColumnSizePtr   :SQLUINTEGER;
 Var DecimalDigitsPtr:SQLSMALLINT;
 Var NullablePtr     :SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLGetData(
     StatementHandle :SQLHSTMT;
     ColumnNumber    :SQLUSMALLINT;
     TargetType      :SQLSMALLINT;
     TargetValuePtr  :SQLPOINTER;
     BufferLength    :SQLINTEGER;
 Var StrLen_or_IndPtr{:SQLINTEGER}
):SQLRETURN; stdcall;


Function SQLBindCol(
     StatementHandle :SQLHSTMT;
     ColumnNumber    :SQLUSMALLINT;
     TargetType      :SQLSMALLINT;
     TargetValuePtr  :SQLPOINTER;
     BufferLength    :SQLINTEGER;
 Var StrLen_or_IndPtr{:SQLINTEGER}
):SQLRETURN; stdcall;

// Parms ---------------------------------------------------------
Const
 SQL_PARAM_INPUT       =1;
 SQL_PARAM_INPUT_OUTPUT=2;
 SQL_PARAM_OUTPUT      =4;

Function SQLNumParams(
     StatementHandle  :SQLHSTMT;
 var ParameterCountPtr:SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLDescribeParam(
     StatementHandle  :SQLHSTMT;
     ParameterNuber   :SQLUSMALLINT;
 var DataTypePrt      :SQLSMALLINT;
 var ParameterSizePtr :SQLUINTEGER;
 var DecimalDigitsPtr :SQLSMALLINT;
 var NullablePtr      :SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLBindParameter(
     StatementHandle :SQLHSTMT;
     ParameterNumber :SQLUSMALLINT;
     InputOutputType :SQLSMALLINT;
     ValueType       :SQLSMALLINT;
     ParameterType   :SQLSMALLINT;
     ColumnSize      :SQLINTEGER;
     DecimalDigits   :SQLSMALLINT;
     ParametrValuePtr:SQLPOINTER;
     BufferLength    :SQLINTEGER;
 Var StrLen_or_IndPtr{:SQLINTEGER}
):SQLRETURN; stdcall;

// Cursor --------------------------------------------------------
Function SQLSetCursorName(
     StatementHandle:SQLHSTMT;
     CursorName     :SQLCHAR;
     NameLength     :SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLCloseCursor(
     StatementHandle:SQLHSTMT
):SQLRETURN; stdcall;

// Sql ----------------------------------------------------------
Const
 SQL_FETCH_NEXT     = 1;
 SQL_FETCH_FIRST    = 2;
 SQL_FETCH_LAST     = 3;
 SQL_FETCH_PRIOR    = 4;
 SQL_FETCH_ABSOLUTE = 5;
 SQL_FETCH_RELATIVE = 6;

 SQL_POSITION = 0;
 SQL_REFRESH  = 1;
 SQL_UPDATE   = 2;
 SQL_DELETE   = 3;

 SQL_LOCK_NO_CHANGE = 0;
 SQL_LOCK_EXCLUSIVE = 1;
 SQL_LOCK_UNLOCK    = 2;

Function SQLPrepare(
     StatementHandle:SQLHSTMT;
     StatementText  :SQLCHAR;
     TextLength     :SQLINTEGER
):SQLRETURN; stdcall;

Function SQLExecute(
     StatementHandle:SQLHSTMT
):SQLRETURN; stdcall;

Function SQLExecDirect(
     StatementHandle:SQLHSTMT;
     StatementText  :SQLCHAR;
     TextLength     :SQLINTEGER
):SQLRETURN; stdcall;

Function SQLNumResultCols(
     StatementHandle:SQLHSTMT;
 var ColumnCountPtr :SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLFetch(
     StatementHandle:SQLHSTMT
):SQLRETURN; stdcall;

Function SQLFetchScroll(
     StatementHandle :SQLHSTMT;
     FetchOrientation:SQLSMALLINT;
     FetchOffset     :SQLINTEGER
):SQLRETURN; stdcall;

Function SQLSetPos(
     StatementHandle :SQLHSTMT;
     RowNumber       :SQLUSMALLINT; // <!> 1..FetchedRows - or - 0 = All the Fetched rows
     Operation       :SQLUSMALLINT;
     LockType        :SQLUSMALLINT
):SQLRETURN; stdcall;

//--------------------------------------------------------------

Function SQLRowCount(
     StatementHandle:SQLHSTMT;
 var RowCountPtr:SQLINTEGER
):SQLRETURN; stdcall;

//--------------------------------------------------------------
const
     { values of NULLABLE field in descriptor }
     SQL_NO_NULLS = 0;
     SQL_NULLABLE = 1;

Function SQLTables(
     StatementHandle:SQLHSTMT;
     CatalogName    :SQLCHAR;
     NameLength1    :SQLSMALLINT;
     SchemaName     :SQLCHAR;
     NameLength2    :SQLSMALLINT;
     TableName      :SQLCHAR;
     NameLength3    :SQLSMALLINT;
     TableType      :SQLCHAR;
     NameLength4    :SQLSMALLINT
):SQLRETURN; stdcall;

Function SQLColumns(
     StatementHandle:SQLHSTMT;
     CatalogName    :SQLCHAR;
     NameLength1    :SQLSMALLINT;
     SchemaName     :SQLCHAR;
     NameLength2    :SQLSMALLINT;
     TableName      :SQLCHAR;
     NameLength3    :SQLSMALLINT;
     ColumnName     :SQLCHAR;
     NameLength4    :SQLSMALLINT
):SQLRETURN; stdcall;

//--------
Const
 ODBC_ADD_DSN            = 1;
 ODBC_CONFIG_DSN         = 2;
 ODBC_REMOVE_DSN         = 3;
 ODBC_ADD_SYS_DSN        = 4;
 ODBC_CONFIG_SYS_DSN     = 5;
 ODBC_REMOVE_SYS_DSN     = 6;
 ODBC_REMOVE_DEFAULT_DSN = 7;

Function SQLManageDataSources(Handle:integer):boolean; stdcall;
Function SQLConfigDataSource(Handle:integer;Request:word;Driver,Attributes:PChar):boolean; stdcall;

implementation

Const
 ODBC_DLL='ODBC32.DLL';
 ODBC_STP='ODBCCP32.DLL';

Function SQLGetDiagRec;    stdcall; external ODBC_DLL;

Function SQLAllocHandle;   stdcall; external ODBC_DLL;
Function SQLFreeHandle;    stdcall; external ODBC_DLL;
Function SQLFreeStmt;      stdcall; external ODBC_DLL;

Function SQLSetEnvAttr;    stdcall; external ODBC_DLL;
Function SQLSetConnectAttr;stdcall; external ODBC_DLL;

Function SQLDriverConnect; stdcall; external ODBC_DLL;
Function SQLDisconnect;    stdcall; external ODBC_DLL;

Function SQLSetStmtAttr;   stdcall; external ODBC_DLL;

Function SQLDescribeCol;   stdcall; external ODBC_DLL;
Function SQLGetData;       stdcall; external ODBC_DLL;
Function SQLBindCol;       stdcall; external ODBC_DLL;

Function SQLNumParams;     stdcall; external ODBC_DLL;
Function SQLDescribeParam; stdcall; external ODBC_DLL;
Function SQLBindParameter; stdcall; external ODBC_DLL;

Function SQLSetCursorName; stdcall; external ODBC_DLL;
Function SQLCloseCursor;   stdcall; external ODBC_DLL;

Function SQLPrepare;       stdcall; external ODBC_DLL;
Function SQLExecute;       stdcall; external ODBC_DLL;
Function SQLExecDirect;    stdcall; external ODBC_DLL;
Function SQLNumResultCols; stdcall; external ODBC_DLL;
Function SQLFetch;         stdcall; external ODBC_DLL;
Function SQLFetchScroll;   stdcall; external ODBC_DLL;
Function SQLSetPos;        stdcall; external ODBC_DLL;

Function SQLRowCount;      stdcall; external ODBC_DLL;

function SQLTables;        stdcall; external ODBC_DLL;
Function SQLColumns;       stdcall; external ODBC_DLL;

Function SQLManageDataSources; stdcall; external ODBC_STP;
Function SQLConfigDataSource;  stdcall; external ODBC_STP;

// ODBC Initialization-----------------------------------
Initialization
 if SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,ODBCHandle) in [0..1] then begin
  SQLSetEnvAttr(ODBCHandle,SQL_ATTR_ODBC_VERSION, SQLPOINTER( SQL_OV_ODBC3), 0);
 end else begin
  ODBCHandle:=SQL_NULL_HANDLE;
 end;

Finalization
 if ODBCHandle<>SQL_NULL_HANDLE then SQLFreeHandle(SQL_HANDLE_ENV,ODBCHandle);

end.
